@@ -51,14 +51,20 @@ class Downloader(AbstractDownloader):
             'proxies': request.proxies and {"http": request.proxies, "https": request.proxies},  # noqa
             'verify': request.options.get("verify", False),
             'impersonate': request.options.get("impersonate") or self.impersonate,
+            'http_version': request.options.get("http_version"),
         }
 
         next_url = request.real_url
         _redirect_count = 0
 
+        if request.use_session:
+            session = request.get_options("$session") or self.get_session()
+        else:
+            session = requests
+
         while True:
             assert _redirect_count < 999, "已经超过最大重定向次数: 999"
-            response = requests.request(**{**options, "url": next_url})
+            response = session.request(**{**options, "url": next_url})
             last_url, next_url = next_url, response.headers.get('location') or response.headers.get('Location')
             if request.allow_redirects and next_url:
                 next_url = urllib.parse.urljoin(response.url, next_url)
@@ -77,7 +83,7 @@ class Downloader(AbstractDownloader):
                         )
                     )
                 )
-                request.options.get('auto_referer', True) and options['headers'].update(Referer=response.url)
+                request.options.get('$referer', False) and options['headers'].update(Referer=response.url)
 
             else:
                 res.content = response.content
@@ -88,3 +94,14 @@ class Downloader(AbstractDownloader):
                 res.request = request
 
                 return res
+
+    def make_session(self) -> requests.Session:
+        return requests.Session()
+
+
+if __name__ == '__main__':
+    downloader = Downloader()
+    rsp = downloader.fetch(Request(url="https://httpbin.org/cookies/set?freeform=123", use_session=True))
+    print(rsp.cookies)
+    rsp = downloader.fetch(Request(url="https://httpbin.org/cookies", use_session=True))
+    print(rsp.text)
