@@ -21,7 +21,7 @@ class Request:
             url: str,
             params: Optional[dict] = None,
             method: str = 'GET',
-            body: Union[str, dict] = None,
+            body: Optional[Union[str, dict]] = None,
             headers: Union[Header, dict] = None,
             cookies: Dict[str, str] = None,
             options: dict = None,
@@ -29,11 +29,13 @@ class Request:
             allow_redirects: bool = True,
             proxies: Optional[str] = None,
             proxy: Optional[dict] = None,
-            status_codes: Optional[dict] = ...,
+            ok: Optional[str] = ...,
             retry: int = 0,
-            max_retry: int = 5,
+            max_retry: [int, float] = 5,
+            use_session: bool = False,
     ) -> None:
         self.url = url
+        self.use_session = use_session
         self.params = params
         self.cookies = cookies
         self.method = method.upper()
@@ -44,7 +46,7 @@ class Request:
         self.allow_redirects = allow_redirects
         self.proxies = proxies
         self.proxy = proxy
-        self.status_codes = status_codes
+        self.ok = ok
         self.retry = retry
         self.max_retry = max_retry
 
@@ -85,20 +87,24 @@ class Request:
         # 开始构建curl命令
         parts = ["curl", f"-X {self.method.upper()}"]
 
-        # 添加请求方法
-
         # 添加请求头
-        if self.headers:
-            for header, value in self.headers.items():
-                parts.append(f"-H {shlex.quote(f'{header}: {value}')}")
 
-        # 添加请求头
+        if self.cookies:
+            cookie_str = "; ".join([f'{k}={v}' for k, v in self.cookies.items()])
+        else:
+            cookie_str = ""
+
         content_type = None
         if self.headers:
             for header, value in self.headers.items():
-                parts.append(f"-H {shlex.quote(f'{header}: {value}')}")
-                if header.lower() == 'content-type':
-                    content_type = value
+                if header.lower() == 'cookie':
+                    cookie_str = "; ".join(list(filter(None, [value, cookie_str])))
+                else:
+                    if header.lower() == 'content-type':
+                        content_type = value.lower()
+                    parts.append(f"-H {shlex.quote(f'{header}: {value}')}")
+
+        cookie_str and parts.append(f"-H {shlex.quote(f'Cookie: {cookie_str}')}")
 
         # 添加请求体
         if self.body:
@@ -189,7 +195,7 @@ class Request:
                 for key in cookie:
                     cookie_dict[key] = cookie[key].value
             else:
-                quoted_headers[header_key] = header_value.strip()
+                quoted_headers[header_key] = str(header_value).strip()
 
         # 解析原始URL
         parsed_url = urllib.parse.urlparse(parsed_args.url)
