@@ -211,7 +211,7 @@ class Response:
         :return:
         """
 
-        return pandora.single(self.xpath(xpath, obj, **kwargs), default=default)
+        return pandora.first(self.xpath(xpath, obj, **kwargs), default=default)
 
     def jsonpath(self, jpath, obj=None, strict=True, **kwargs):
         """
@@ -243,7 +243,7 @@ class Response:
         :param kwargs:
         :return:
         """
-        return pandora.single(self.jsonpath(jpath, obj, strict, **kwargs), default=default)
+        return pandora.first(self.jsonpath(jpath, obj, strict, **kwargs), default=default)
 
     def re(self, regex, obj=None, **kwargs):
         """
@@ -275,7 +275,7 @@ class Response:
         :param kwargs:
         :return:
         """
-        return pandora.single(self.re(regex, obj, **kwargs), default=default)
+        return pandora.first(self.re(regex, obj, **kwargs), default=default)
 
     def get(self, rule: str, obj=None, strict=True, **kwargs):
         """
@@ -313,7 +313,7 @@ class Response:
         :return:
 
         """
-        return pandora.single(self.get(rule, obj, strict, **kwargs), default=default)
+        return pandora.first(self.get(rule, obj, strict, **kwargs), default=default)
 
     @property
     def ok(self):
@@ -322,16 +322,14 @@ class Response:
 
         :return:
         """
-        status_codes = self.request.status_codes
+        ok = self.request.ok
 
         # None -> 所有状态码, 除了 -1 , -1 代表请求过程中发生了错误
-        if status_codes is None:
+        if ok is None:
             return self.status_code != -1
 
-        elif isinstance(status_codes, dict):
-            mode = status_codes.get('mode', 'include')
-            default = 1 if mode == 'include' else 0
-            return bool(status_codes.get(self.status_code, default))
+        elif isinstance(ok, str):
+            return eval(ok, {"status_code": self.status_code, "response": self})
 
         # 默认 -> 200 到 400
         else:
