@@ -2,25 +2,31 @@
 # @Time    : 2023-11-15 14:09
 # @Author  : Kem
 # @Desc    :
+import asyncio
+import collections
+import contextlib
 import datetime
 import functools
 import inspect
+import itertools
 import math
 import queue
 import re
 import time
-from typing import Optional, Union, Iterable, Callable, List
+import uuid
+from typing import Optional, Union, Iterable, Callable, List, Generator
 
 from loguru import logger
 
 from bricks import state
 from bricks.core import dispatch, signals, events
-from bricks.core.context import Flow
+from bricks.core.context import Flow, Error
+from bricks.core.events import EventManager
 from bricks.core.genesis import Pangu
 from bricks.downloader import cffi, AbstractDownloader
 from bricks.lib.counter import FastWriteCounter
 from bricks.lib.items import Items
-from bricks.lib.proxies import manager
+from bricks.lib.proxies import manager, BaseProxy
 from bricks.lib.queues import TaskQueue, LocalQueue, Item
 from bricks.lib.request import Request
 from bricks.lib.response import Response
@@ -38,11 +44,11 @@ class Context(Flow):
             **kwargs
 
     ) -> None:
-        self.request: Request = kwargs.pop("request", None)
-        self.response: Response = kwargs.pop("response", None)
-        self.seeds: Union[Item, List[Item]] = kwargs.pop("seeds", None)
-        self.items: Items = kwargs.pop("items", None)
-        self.task_queue: TaskQueue = kwargs.pop("task_queue", None)
+        self.request: Optional[Request] = kwargs.pop("request", None)
+        self.response: Optional[Response] = kwargs.pop("response", None)
+        self.seeds: Optional[Union[Item, List[Item]]] = kwargs.pop("seeds", None)
+        self.items: Optional[Items] = kwargs.pop("items", None)
+        self.task_queue: Optional[TaskQueue] = kwargs.pop("task_queue", None)
         self.queue_name: str = kwargs.pop("queue_name", f'{self.__class__.__module__}.{self.__class__.__name__}')
         super().__init__(form, target, **kwargs)
         self.target: Spider = target
@@ -81,7 +87,7 @@ class Context(Flow):
 
         :return:
         """
-        ret = self.task_queue.replace(self.queue_name, self.seeds, new, qtypes=qtypes)
+        ret = self.task_queue.replace(self.queue_name, (self.seeds, new), qtypes=qtypes)
         self.seeds.fingerprint = new
         return ret
 
@@ -105,14 +111,21 @@ class Context(Flow):
                 ret.append(stuff)
         else:
             if call_later:
-                self.task_queue.put(self.queue_name, *obj)
+                qtypes = "current"
             else:
-                self.task_queue.put(self.queue_name, *obj, qtypes="temp")
+                qtypes = "temp"
+
+            self.target.put_seeds(
+                obj,
+                task_queue=self.task_queue,
+                queue_name=self.queue_name,
+                qtypes=qtypes
+            )
 
         return ret
 
     def clear_proxy(self):
-        manager.clear_proxy(self.request.proxy or self.target.proxy)
+        manager.clear(self.request.proxy or self.target.proxy)
         self.request.proxies = None
 
     error = failure
@@ -127,9 +140,11 @@ class InitContext(Flow):
             **kwargs
 
     ) -> None:
-        self.seeds: List[Item] = kwargs.pop("seeds", None)
-        self.task_queue: TaskQueue = kwargs.pop("task_queue", None)
+        self.seeds: Optional[List[Item]] = kwargs.pop("seeds", None)
+        self.task_queue: Optional[TaskQueue] = kwargs.pop("task_queue", None)
         self.queue_name: str = kwargs.pop("queue_name", f'{self.__class__.__module__}.{self.__class__.__name__}')
+        self.maxsize: Optional[int] = kwargs.pop("maxsize", None)
+        self.priority: bool = kwargs.pop("priority", False)
         super().__init__(form, target, **kwargs)
         self.target: Spider = target
 
@@ -163,7 +178,7 @@ class Spider(Pangu):
             downloader: Optional[Union[str, AbstractDownloader]] = None,
             task_queue: Optional[TaskQueue] = None,
             queue_name: Optional[str] = "",
-            proxy: Optional[dict] = None,
+            proxy: Optional[Union[dict, BaseProxy]] = None,
             forever: Optional[bool] = False,
             **kwargs
     ) -> None:
@@ -177,6 +192,15 @@ class Spider(Pangu):
         super().__init__(**kwargs)
         self.number_of_total_requests = FastWriteCounter()  # 发起请求总数量
         self.number_of_failure_requests = FastWriteCounter()  # 发起请求失败数量
+        self.number_of_new_seeds = FastWriteCounter()  # 动态新增的种子数量(翻页/拆分等等)
+        self.number_of_seeds_obtained = FastWriteCounter()  # 获取得到的种子数量
+        self.number_of_seeds_pending = 0  # 待处理的种子数量
+
+    is_master = property(
+        fget=lambda self: getattr(self, "$isMaster", False),
+        fset=lambda self, v: setattr(self, "$isMaster", v),
+        fdel=lambda self: setattr(self, "$isMaster", False),
+    )
 
     @property
     def flows(self):
@@ -204,6 +228,7 @@ class Spider(Pangu):
             logger.debug(f"[停止投放] 当前机器 ID: {state.MACHINE_ID}, 原因: {pinfo['msg']}")
             return
 
+        self.is_master = True
         task_queue.command(queue_name, {"action": task_queue.COMMANDS.SET_INIT})
 
         logger.debug(f"[开始投放] 获取初始化权限成功, MACHINE_ID: {state.MACHINE_ID}")
@@ -303,13 +328,13 @@ class Spider(Pangu):
         context.seeds = seeds
         fettle = pandora.invoke(
             func=self.put_seeds,
+            args=[context.seeds],
             kwargs={
-                "seeds": seeds,
                 "maxsize": settings['queue_size'],
-                "where": "init",
                 "task_queue": context.task_queue,
                 "queue_name": context.queue_name,
-
+                "priority": context.priority,
+                "where": "init"
             },
             annotations={InitContext: context},
             namespace={"context": context},
@@ -342,38 +367,76 @@ class Spider(Pangu):
         else:
             context.flow()
 
-    def put_seeds(self, **kwargs):
+    def put_seeds(
+            self,
+            seeds: Union[dict, Item, Iterable[Item], Iterable[dict]],
+            task_queue: Optional[TaskQueue] = ...,
+            queue_name: Optional[str] = ...,
+            maxsize: Optional[int] = None,
+            priority: Optional[bool] = False,
+            **kwargs
+    ):
         """
         将种子放入容器
 
-        :param kwargs:
+        :param priority: 种子投放优先级, 需要队列支持
+        :param maxsize: 种子限制数量, 到达这个数量会被阻塞
+        :param queue_name: 队列名
+        :param task_queue: 队列
+        :param seeds: 种子
+        :param kwargs: 其他参数
         :return:
         """
-        task_queue: TaskQueue = kwargs.pop('task_queue', None) or self.task_queue
-        queue_name: str = kwargs.pop('queue_name', None) or self.queue_name
-        seeds = kwargs.pop('seeds', {})
 
-        maxsize = kwargs.pop('maxsize', None)
-        priority = kwargs.pop('priority', False)
+        if task_queue is ...: task_queue = self.task_queue
+        if queue_name is ...: queue_name = self.queue_name
+        seeds = seeds or {}
         task_queue.continue_(queue_name, maxsize=maxsize, interval=1)
         return task_queue.put(queue_name, *pandora.iterable(seeds), priority=priority, **kwargs)
 
     def _when_put_seeds(self, raw_method):  # noqa
         @functools.wraps(raw_method)
-        def wrapper(*args, **kwargs):
-            context: InitContext = InitContext.get_context()
-            context.form = state.const.BEFORE_PUT_SEEDS
-            events.EventManager.invoke(context)
-            prepared = pandora.prepare(
-                func=raw_method,
-                args=args,
-                kwargs=kwargs,
-                annotations={InitContext: context},
-                namespace={"context": context}
-            )
+        def wrapper(
+                seeds: Union[dict, Item, List[Item], List[dict]],
+                task_queue: Optional[TaskQueue] = ...,
+                queue_name: Optional[str] = ...,
+                maxsize: Optional[int] = None,
+                priority: Optional[bool] = False,
+                **kwargs
+        ):
+            where = kwargs.pop('where', None)
+            if where == 'init':
+                context: InitContext = InitContext.get_context()
+                context.form = state.const.BEFORE_PUT_SEEDS
+                events.EventManager.invoke(context)
+                args = [context.seeds]
+                kws = {
+                    "task_queue": context.task_queue,
+                    "queue_name": context.queue_name,
+                    "maxsize": context.maxsize,
+                    "priority": context.priority,
+                    **kwargs
+                }
+            else:
+                args = [seeds]
+                kws = {
+                    "task_queue": task_queue,
+                    "queue_name": queue_name,
+                    "maxsize": maxsize,
+                    "priority": priority,
+                    **kwargs
+                }
+
+            prepared = pandora.prepare(func=raw_method, args=args, kwargs=kws)
             ret = prepared.func(*prepared.args, **prepared.kwargs)
-            context.form = state.const.AFTER_PUT_SEEDS
-            events.EventManager.invoke(context)
+
+            if where == 'init':
+                context: InitContext = InitContext.get_context()
+                context.form = state.const.AFTER_PUT_SEEDS
+                events.EventManager.invoke(context)
+            else:
+                self.number_of_new_seeds.increment(ret)
+
             return ret
 
         return wrapper
@@ -385,7 +448,6 @@ class Spider(Pangu):
 
         :return:
         """
-        count = 0
         task_queue: TaskQueue = self.get("spider.task_queue", self.task_queue)
         queue_name: str = self.get("spider.queue_name", self.queue_name)
         output = time.time()
@@ -439,9 +501,37 @@ class Spider(Pangu):
                             self.dispatcher.running == 0 and
                             task_queue.is_empty(queue_name, threshold=self.get("spider.threshold", default=0))
                     ):
+
+                        number_of_seeds_obtained = self.number_of_seeds_obtained.value
+                        number_of_new_seeds = self.number_of_new_seeds.value
+                        number_of_total_requests = self.number_of_total_requests.value
+                        number_of_failure_requests = self.number_of_failure_requests.value
+                        number_of_success_requests = number_of_total_requests - number_of_failure_requests
+                        if number_of_total_requests:
+                            rate_of_success_requests = round(
+                                number_of_success_requests / number_of_total_requests * 100, 2
+                            )
+                        else:
+                            rate_of_success_requests = 0
+
                         logger.debug(
-                            f'[爬取完毕] 队列名称: {queue_name}, 关闭阈值: {self.get("spider.threshold", default=0)}, 提交种子的数量: {count}')
-                        return count
+                            f'[爬取完毕] '
+                            f'队列名称: {queue_name} '
+                            f'关闭阈值: {self.get("spider.threshold", default=0)} '
+                            f'已获取的种子数量: {number_of_seeds_obtained} '
+                            f'新增的种子数量: {number_of_new_seeds} '
+                            f'总的请求数量: {number_of_total_requests} '
+                            f'请求成功率: {rate_of_success_requests}% '
+                        )
+
+                        return {
+                            "number_of_seeds_obtained": number_of_seeds_obtained,
+                            "number_of_new_seeds": number_of_new_seeds,
+                            "number_of_total_requests": number_of_total_requests,
+                            "number_of_failure_requests": number_of_failure_requests,
+                            "number_of_success_requests": number_of_success_requests,
+                            "request_success_rate": rate_of_success_requests,
+                        }
 
                     else:
 
@@ -455,12 +545,20 @@ class Spider(Pangu):
 
                             time.sleep(1)
 
+                except (KeyboardInterrupt, SystemExit):
+                    raise
+
+                except Exception as e:
+                    EventManager.invoke(Error(context=context, error=e), errors="output")
+
                 else:
+                    self.number_of_seeds_pending += len(pandora.iterable(fettle))
                     for seeds in pandora.iterable(fettle):
                         stuff = context.copy()
                         stuff.flow({"next": self.on_consume, "seeds": seeds})
                         self.submit(dispatch.Task(stuff.next.root, [stuff]))
-                        count += 1
+                        self.number_of_seeds_obtained.increment()
+                        self.number_of_seeds_pending -= 1
 
     def _when_run_spider(self, raw_method):
         @functools.wraps(raw_method)
@@ -468,15 +566,20 @@ class Spider(Pangu):
             with self.dispatcher:
                 task_queue: TaskQueue = self.get("spider.task_queue", self.task_queue)
                 queue_name: str = self.get("spider.queue_name", self.queue_name)
-                task_queue.command(
+                server = task_queue.command(
                     queue_name,
                     {
                         "action": task_queue.COMMANDS.RUN_SUBSCRIBE,
-                        "target": self
+                        "target": {
+                            "collect-status": lambda: self.dispatcher.running + self.number_of_seeds_pending
+                        }
                     }
                 )
 
-                return raw_method(*args, **kwargs)
+                try:
+                    return raw_method(*args, **kwargs)
+                finally:
+                    hasattr(server, "stop") and server.stop()
 
         return wrapper
 
@@ -484,11 +587,14 @@ class Spider(Pangu):
         with self.dispatcher:
             t1 = int(time.time() * 1000)
             init_task = dispatch.Task(func=self.run_init)
-            self.active(init_task)
+            future = self.active(init_task)
             task_queue: TaskQueue = self.get("init.task_queue", self.task_queue)
             queue_name: str = self.get("init.queue_name", self.queue_name)
             task_queue.command(queue_name, {"action": task_queue.COMMANDS.WAIT_INIT, "time": t1})
-            self.run_spider()
+            return {
+                "spider": self.run_spider(),
+                "init": future.result()
+            }
 
     @staticmethod
     def get_seeds(**kwargs) -> Union[Iterable[Item], Item]:
@@ -540,11 +646,12 @@ class Spider(Pangu):
         self.number_of_failure_requests.increment()
         request: Request = context.request
         response: Response = context.response
+        error: str = response.error if response else ""
 
         if request.retry < request.max_retry - 1:
 
             # 如果是代理错误, 则不计算重试次数
-            if not IGNORE_RETRY_PATTERN.search(response.error):
+            if not IGNORE_RETRY_PATTERN.search(error):
                 request.retry += 1
 
             # 保留代理标志
@@ -576,7 +683,7 @@ class Spider(Pangu):
                 args=args,
                 kwargs=kwargs,
                 annotations={
-                    Context: context,
+                    self.Context: context,
                     Response: context.response,
                     Item: context.seeds
                 },
@@ -624,7 +731,7 @@ class Spider(Pangu):
                 args=args,
                 kwargs=kwargs,
                 annotations={
-                    Context: context,
+                    self.Context: context,
                     Item: context.seeds,
                     Request: context.request,
                 },
@@ -781,13 +888,22 @@ class Spider(Pangu):
         raise NotImplementedError
 
     def make_request(self, context: Context) -> Request:
-        raise NotImplementedError
+        return pandora.invoke(Request, kwargs=context.seeds)
 
-    def parse(self, context: Context):
-        raise NotImplementedError
+    def parse(self, context: Context) -> Union[List[dict], Items]:
+        return Items({
+            "request": context.request.curl,
+            "response": {
+                "url": context.response.url,
+                "headers": context.response.headers,
+                "status_code": context.response.status_code,
+                "text": context.response.text,
+            }
+        })
 
     def item_pipeline(self, context: Context):
         context.items and logger.debug(context.items)
+        context.success()
 
     def install(self):
         super().install()
@@ -804,7 +920,13 @@ class Spider(Pangu):
         )
 
     @pandora.Method
-    def survey(self_or_cls, *seeds: dict, attrs: dict = None, modded: dict = None) -> List[Context]:  # noqa
+    def survey(
+            binding,  # noqa
+            *seeds: dict,
+            attrs: dict = None,
+            modded: dict = None,
+            extract: list = None
+    ) -> List[Context]:
         """
         调查种子, collect 会收集产生的 Context
         用户可以从 collect 的结果中根据 Context 获取到当时的 response, items, request, seeds 等等
@@ -812,6 +934,7 @@ class Spider(Pangu):
         但是: 如果在运行过程中用户修改了里面的结果, 那会被覆盖掉
         survey 会屏蔽原来的 make_seeds 和 item_pipeline 方法, 会使用用户传入的 seeds, 并且仅仅输出结果 (不会存储)
 
+        :param extract: 
         :param modded: 魔改 class
         :param attrs: 初始化参数
         :param seeds: 需要调查的种子
@@ -819,14 +942,15 @@ class Spider(Pangu):
         """
         attrs = attrs or {}
         modded = modded or {}
+        extract = extract or []
 
-        if isinstance(self_or_cls, type):
-            cls = self_or_cls
+        if isinstance(binding, type):
+            cls = binding
         else:
-            cls = self_or_cls.__class__
-            attrs.setdefault("proxy", self_or_cls.proxy)
-            attrs.setdefault("concurrency", self_or_cls.concurrency)
-            attrs.setdefault("downloader", self_or_cls.downloader)
+            cls = binding.__class__
+            attrs.setdefault("proxy", binding.proxy)
+            attrs.setdefault("concurrency", binding.concurrency)
+            attrs.setdefault("downloader", binding.downloader)
 
         collect = queue.Queue()
 
@@ -849,8 +973,219 @@ class Spider(Pangu):
         clazz = type("Survey", (cls,), modded)
         survey: Spider = clazz(**attrs)
         survey.run()
-        return list(collect.queue)
+        return list(collect.queue) if not extract else [{k: getattr(c, k) for k in extract} for c in collect.queue]
+
+    @pandora.Method
+    def listen(
+            binding,  # noqa
+            attrs: dict = None,
+            modded: dict = None,
+    ) -> 'Listener':
+        """
+        将 Spider 转化为 Listener, 等于后台运行爬虫, 支持动态添加种子, 然后获取该种子消耗的结果
+        实现类似将爬虫转化为 API 的效果, 避免维护多份代码
+
+        但是, 部分事件会失效:
+        1. 如果只要 response, 那么生效的事件只有: before request, after request
+        2. 如果只要 request, 那么生效的事件只有: before request
+        3. 如果只要 items, 那么生效的事件只有: before request, after request
+
+        也就是说: 如果你有翻页事件, 且在 pipeline, 则不会生效 (但是我们一般都不需要这个, 建议自己注释掉)
+
+        :param modded: 魔改 class
+        :param attrs: 初始化参数
+        :return:
+        """
+        attrs = attrs or {}
+        modded = modded or {}
+
+        if isinstance(binding, type):
+            cls = binding
+        else:
+            cls = binding.__class__
+            attrs.setdefault("proxy", binding.proxy)
+            attrs.setdefault("concurrency", binding.concurrency)
+            attrs.setdefault("downloader", binding.downloader)
+
+        def mock_on_success(self, shutdown=False):
+            future_id = self.seeds.get('$futureID')
+            if future_id in listener.futures:
+                future: Optional[queue.Queue] = listener.futures[future_id]
+                future.put(self)
+
+            return super(self.__class__, self).success(shutdown)
+
+        def mock_on_retry(self):
+            future_id = self.seeds.get('$futureID')
+            future_max_retry = self.seeds.get('$futureMaxRetry')
+            counter = listener.counter[future_id]
+            times = next(counter)
+            if times + 1 >= future_max_retry:
+                listener.counter.pop(future_id, None)
+                return self.success(shutdown=True)
+            else:
+                return super(self.__class__, self).retry()
+
+        def mock_on_request(self, context: Context):
+            future_type = context.seeds.get('$futureType', "$response")
+            if future_type == '$request':
+                raise signals.Success
+            return super(self.__class__, self).on_request(context)
+
+        def mock_on_response(self, context: Context):
+            future_type = context.seeds.get('$futureType', "$response")
+
+            if future_type == '$response':
+                raise signals.Success
+
+            items: Items = super(self.__class__, self).mock_on_response(context)
+
+            if future_type == '$items':
+                raise signals.Success
+
+            return items
+
+        modded.setdefault("on_request", mock_on_request)
+        modded.setdefault("on_response", mock_on_response)
+        local = LocalQueue()
+        attrs.update({
+            "task_queue": local,
+            "spider.task_queue": local,
+            "queue_name": f"{cls.__module__}.{cls.__name__}:listen",
+            "forever": True,
+        })
+        clazz = type("Listen", (cls,), modded)
+
+        clazz.Context = type("ListenContext", (cls.Context,), {"success": mock_on_success, "retry": mock_on_retry})
+        listen: Spider = clazz(**attrs)
+        listener = Listener(listen)
+        return listener.run()
+
+    def fetch(self, request: [Request, dict], options: dict = None) -> Response:
+        """
+        发送请求获取响应
+
+        默认情况下, 只要response 的状态码不为-1( 框架内部错误/ 异常) 就会结束
+        如果失败五次, 也会结束, 所以需要一定成功可以将 request.max_retry = math.inf
+
+        :param request:
+        :param options: 其他选项, 覆盖 downloader / proxy 的时候可以使用
+        :return:
+        """
+        if isinstance(request, dict):
+            request = Request(**request)
+
+        if request.ok is ...:
+            request.ok = 'response.status_code != -1'
+
+        dispatcher = contextlib.nullcontext()
+        options = options or {}
+        plugins: Union[dict, type(...), None] = options.pop("plugins", ...)
+        options.setdefault("downloader", self.downloader)
+        options.setdefault("proxy", self.proxy)
+        spider = Spider(**options)
+
+        # 不需要任何插件
+        if not plugins:
+            for plugin in spider.plugins:
+                plugin.unregister()
+
+        # 使用默认插件
+        elif plugins is ...:
+            pass
+
+        else:
+            for plugin in spider.plugins:
+                plugin.unregister()
+            for form, plugin in plugins:
+                spider.use(form, *pandora.iterable(plugin))
+
+        if inspect.iscoroutinefunction(spider.downloader.fetch):
+            dispatcher = spider.dispatcher
+
+        with dispatcher:
+            context = spider.make_context(
+                request=request,
+                next=spider.on_request,
+                flows={
+                    spider.on_request: None,
+                    spider.on_retry: spider.on_request
+                },
+            )
+            context.failure = lambda shutdown: context.flow({"next": None})
+            spider.on_consume(context=context)
+            return context.response
 
 
-if __name__ == '__main__':
-    Spider().survey({"id": 1})
+class Listener:
+
+    def __init__(self, spider: Spider):
+        self.spider: Spider = spider
+        self.futures = collections.defaultdict(queue.Queue)
+        self.counter = collections.defaultdict(itertools.count)
+
+    def run(self):
+        self.spider.dispatcher.start()
+        self.spider.active(dispatch.Task(func=self.spider.run, kwargs={"task_name": "spider"}))
+        return self
+
+    def stop(self):
+        self.spider.forever = False
+
+    def listen(self, seeds: Union[dict, Item], timeout: int = None) -> Generator[Context, Union[dict, Item], None]:
+        """
+        给 listener 一个种子, 然后获取种子的消耗结果
+        种子内特殊键值对说明:
+        $futureID 当前任务 ID, 自动生成
+
+        $futureType 表示需要的类型, 可以自己设置, 默认为 $response
+            $request -> 表示只需要 request, 也就是消耗到了请求之前就会告知结果
+            $response -> 表示只要 response, 也就是消耗到了解析之前就会告知结果
+            $items -> 表示只要 items, 也就是消耗到了存储之前就会告知结果
+
+
+        :param seeds: 需要消耗的种子
+        :param timeout: 超时时间, 超时后还没有获取到结果则退出, 如果为 None 则表示必须等待一个结果才会结束
+        :return:
+        """
+        future_id = str(uuid.uuid4())
+        future = self.futures[future_id]
+        seeds.update({"$futureID": future_id})
+        self.spider.put_seeds(seeds, task_queue=self.spider.get("spider.task_queue"))
+        try:
+            if timeout is None:
+                loop = range(1)
+            else:
+                loop = itertools.cycle([None])
+
+            for _ in loop:
+                yield future.get(timeout=timeout)
+        except GeneratorExit:
+            raise
+        except queue.Empty:
+            return
+        finally:
+            self.futures.pop(future_id, None)
+
+    async def wait(self, seeds: Union[dict, Item], timeout: int = None) -> Generator[Context, None, None]:
+        """
+        listen 的 异步实现
+
+        :param seeds:
+        :param timeout:
+        :return:
+        """
+        loop = asyncio.get_event_loop()
+        sync_gen = self.listen(seeds, timeout=timeout)
+
+        def next_item():
+            try:
+                return next(sync_gen)
+            except StopIteration as e:
+                return e
+
+        while True:
+            item = await loop.run_in_executor(None, next_item)
+            if isinstance(item, StopIteration):
+                break
+            yield item
