@@ -16,7 +16,7 @@ from bricks.lib.items import Items
 from bricks.lib.nodes import RenderNode
 from bricks.lib.queues import Item
 from bricks.spider import air
-from bricks.utils import pandora, convert
+from bricks.utils import pandora
 
 
 class Context(air.Context):
@@ -40,15 +40,18 @@ class Context(air.Context):
 
     def get_node(self, signpost=None):
         if signpost is None:
-            cursor = max(0, self.seeds.setdefault('$signpost', 0) - 1)
+            cursor = max(0, self.signpost - 1)
         else:
             cursor = signpost
 
         return self.target.config.spider[cursor]
 
+    def archive(self, signpost: int, **kwargs):
+        return self.replace({**self.seeds, "$signpost": signpost, **kwargs})
 
-class Task(_events.Task, RenderNode):
-    ...
+    @property
+    def signpost(self):
+        return self.seeds.setdefault("$signpost", 0)
 
 
 @dataclass
@@ -59,12 +62,16 @@ class Layout(RenderNode):
     default: dict = None
 
 
+class Task(_events.Task, RenderNode):
+    archive: bool = False
+
+
 @dataclass
 class Download(RenderNode):
     url: str = ...
     params: Optional[dict] = None
     method: str = 'GET'
-    body: Union[str, dict] = None
+    body: Optional[Union[str, dict]] = None
     headers: Union[Header, dict] = None
     cookies: Dict[str, str] = None
     options: dict = None
@@ -72,9 +79,10 @@ class Download(RenderNode):
     allow_redirects: bool = True
     proxies: Optional[str] = None
     proxy: Optional[dict] = None
-    status_codes: Optional[dict] = ...
+    ok: Optional[str] = ...
     retry: int = 0
     max_retry: int = 5
+    use_session: bool = False
     archive: bool = False
 
     def to_request(self) -> Request:
@@ -90,14 +98,12 @@ class Download(RenderNode):
             allow_redirects=self.allow_redirects,
             proxies=self.proxies,
             proxy=self.proxy,
-            status_codes=self.status_codes,
+            ok=self.ok,
             retry=self.retry,
-            max_retry=self.max_retry
+            max_retry=self.max_retry,
+            use_session=self.use_session
         )
 
-    def to_response(self, options: dict = None) -> Response:
-        return convert.req2resp(self.to_request(), options)
-
 
 @dataclass
 class Parse(RenderNode):
@@ -105,6 +111,7 @@ class Parse(RenderNode):
     args: Optional[list] = None
     kwargs: Optional[dict] = None
     layout: Optional[Layout] = None
+    archive: bool = False
 
 
 @dataclass
@@ -114,6 +121,7 @@ class Pipeline(RenderNode):
     kwargs: Optional[dict] = None
     success: bool = False
     layout: Optional[Layout] = None
+    archive: bool = False
 
 
 @dataclass
@@ -158,7 +166,7 @@ class Spider(air.Spider):
             raise signals.Exit
 
         while True:
-            signpost: int = context.seeds.setdefault('$signpost', 0)
+            signpost: int = context.signpost
             context.seeds.setdefault('$bookmark', 0)
 
             try:
@@ -173,25 +181,28 @@ class Spider(air.Spider):
                 # 种子 -> Request
                 if isinstance(node, Download):
                     if context.next.prev and context.next.prev.root == self.on_retry:
-                        # 这是需要重试的
+                        # 从重试请求那边过来的
                         context.flow({"next": self.on_request})
                     else:
-                        # 这是新的请求
+                        # 将种子 -> request -> 发送请求
                         context.seeds['$bookmark'] = signpost
-                        node.archive and context.replace({**context.seeds, "$signpost": signpost})
+                        node.archive and context.archive(signpost)
                         context.flow({"next": self.make_request})
                     raise signals.Switch
 
                 # Request -> Response
                 elif isinstance(node, Parse):
+                    node.archive and context.archive(signpost)
                     context.flow({"next": self.on_response})
                     raise signals.Switch
 
                 elif isinstance(node, Pipeline):
+                    node.archive and context.archive(signpost)
                     context.flow({"next": self.on_pipeline})
                     raise signals.Switch
 
                 elif isinstance(node, Task):
+                    node.archive and context.archive(signpost)
                     pandora.invoke(
                         func=node.func,
                         args=node.args,
@@ -236,6 +247,7 @@ class Spider(air.Spider):
                 annotations={Context: context},
                 namespace={"context": context}
             )
+            layout = layout.render(seeds)
 
             if inspect.isgenerator(seeds):
                 for seed in seeds:
@@ -264,7 +276,7 @@ class Spider(air.Spider):
         context.flow({"request": request})
         return request
 
-    def parse(self, context: Context):
+    def parse(self, context: Context) -> Union[List[dict], Items]:
         node: Parse = context.get_node()
         engine = node.func
         args = node.args or []
@@ -312,25 +324,14 @@ class Spider(air.Spider):
                 }
             )
 
-        if inspect.isgenerator(items):
-            for item in items:
-                pandora.clean_rows(
-                    *pandora.iterable(item),
-                    rename=layout.rename,
-                    default=layout.default,
-                    factory=layout.factory,
-                    show=layout.show,
-                )
-                yield item
-        else:
-            pandora.clean_rows(
-                *pandora.iterable(items),
-                rename=layout.rename,
-                default=layout.default,
-                factory=layout.factory,
-                show=layout.show,
-            )
-            yield items or []
+        pandora.clean_rows(
+            *pandora.iterable(items),
+            rename=layout.rename,
+            default=layout.default,
+            factory=layout.factory,
+            show=layout.show,
+        )
+        return Items(items or [])
 
     def item_pipeline(self, context: Context):
         node: Pipeline = context.get_node()
@@ -342,7 +343,8 @@ class Spider(air.Spider):
 
         if not callable(engine):
             engine = pandora.load_objects(engine)
-        back = context.items
+
+        backup = context.items
         try:
             context.items = pandora.clean_rows(
                 *copy.deepcopy(context.items),
@@ -372,7 +374,7 @@ class Spider(air.Spider):
                 }
             )
         finally:
-            context.items = back
+            context.items = backup
 
         node.success and context.success()
 
@@ -381,10 +383,3 @@ class Spider(air.Spider):
 
         for form, events in (self.config.events or {}).items():
             self.use(form, *events)
-
-
-if __name__ == '__main__':
-    down = Download(url="http://www.baidu.com", params={"skus": '{sku:allsku}'})
-    down.register_adapter("allsku", lambda has_sku, miss_sku: has_sku + miss_sku)
-    rendered = down.render({"has_sku": [1, 2, 3], "miss_sku": [4, 5, 6]})
-    print(rendered.params)
