@@ -3,7 +3,6 @@
 # @Author  : Kem
 # @Desc    :
 import hashlib
-import os.path
 import uuid
 
 from bricks.lib import variable
@@ -13,16 +12,11 @@ __all__ = (
     "G",
     "T",
     "MACHINE_ID",
-    "VERSION",
 )
 
 # 当前 机器 ID
 MACHINE_ID = hashlib.sha256(uuid.UUID(int=uuid.getnode()).hex[-12:].encode()).hexdigest()
 
-with open(os.path.join(os.path.dirname(__file__), "VERSION"), "r") as f:
-    # 当前框架版本
-    VERSION = f.read().strip()
-
 # 全局变量
 G = variable.VariableG()
 # 线程变量
